#version 150

in  vec3 vPosition;
in  vec4 vNormal;
in  vec2 vTexCoord;
uniform mat4 p,mv;

float shininess = 10;
vec4 ambientLight = vec4(.4,.4,.4,1);
vec4 diffuseLight = vec4(.8,.8,.8,1);
vec4 specularLight = vec4(1,1,1,1);
vec4 lightPosition = vec4(1,1,1,0);

out vec2 TexCoord;
out vec4 color;

void main() 
{
  vec4 mvPosition = mv*vec4(vPosition,1);
  gl_Position = p*mvPosition;
  
  //Set up Normal, Light, Eye and Half vectors
  vec3 N = normalize((mv*vNormal).xyz);
  vec3 L = normalize(lightPosition.xyz - mvPosition.xyz);
  if (lightPosition.w == 0.0) L = normalize(lightPosition.xyz);
  vec3 E = -normalize(mvPosition.xyz);
  vec3 H = normalize(L+E); 

  //Calculate diffuse coefficient
  float Kd = max(dot(L,N), 0.0);

  //Calculate Blinn-Phong specular coefficient
  float Ks = pow(max(dot(N,H), 0.0), shininess);

  //Calculate lit colour for this pixel
  color = Kd * diffuseLight + Ks * specularLight + ambientLight;
  TexCoord = vTexCoord;
} 
